@@ -18,7 +18,9 @@ import io
 import os
 import tempfile
 import time
-import typing  # noqa
+import typing
+from abc import ABC
+from urllib.parse import urlparse
 
 from oslo_log import log
 import requests
@@ -28,7 +30,6 @@ from tobiko.config import get_bool_env
 from tobiko.openstack.glance import _client
 from tobiko.openstack.glance import _io
 from tobiko.openstack import keystone
-from tobiko.shell import sh
 
 
 LOG = log.getLogger(__name__)
@@ -82,14 +83,15 @@ class GlanceImageStatus(object):
 
 
 @keystone.skip_unless_has_keystone_credentials()
-class GlanceImageFixture(_client.HasGlanceClientMixin, tobiko.SharedFixture):
+class GlanceImageFixture(
+        _client.HasGlanceClientMixin, tobiko.SharedFixture, ABC):
 
     image_name: typing.Optional[str] = None
     username: typing.Optional[str] = None
     password: typing.Optional[str] = None
     disabled_algorithms: typing.Optional[typing.Dict[str, typing.Any]] = None
     image = None
-    wait_interval = 5.
+    wait_interval = 3.
 
     def __init__(self,
                  image_name: typing.Optional[str] = None,
@@ -185,7 +187,7 @@ class GlanceImageFixture(_client.HasGlanceClientMixin, tobiko.SharedFixture):
                                            expected_status=expected_status)
 
 
-class UploadGlanceImageFixture(GlanceImageFixture):
+class UploadGlanceImageFixture(GlanceImageFixture, ABC):
 
     disk_format = "raw"
     container_format = "bare"
@@ -296,20 +298,26 @@ class UploadGlanceImageFixture(GlanceImageFixture):
         raise NotImplementedError
 
 
-class FileGlanceImageFixture(UploadGlanceImageFixture):
+class UrlGlanceImageFixture(UploadGlanceImageFixture, ABC):
 
-    image_file = None
-    image_dir = None
-    compression_type = None
+    image_url: str = ''
+    image_dir: str = ''
+    image_file: str = ''
 
-    def __init__(self, image_file=None, image_dir=None, **kwargs):
-        super(FileGlanceImageFixture, self).__init__(**kwargs)
-
-        if image_file:
-            self.image_file = image_file
-        elif not self.image_file:
-            self.image_file = self.fixture_name
-        tobiko.check_valid_type(self.image_file, str)
+    def __init__(self,
+                 image_url: str = None,
+                 image_dir: str = None,
+                 **kwargs):
+        super().__init__(**kwargs)
+        if image_url:
+            self.image_url = image_url
+        tobiko.check_valid_type(self.image_url, str)
+        # self.image_url has to be a URL - if it refers to a local file, it
+        # should start with file://
+        url = urlparse(self.image_url)
+        if not self.image_file:
+            self.image_file = (url.path if url.scheme == 'file'
+                               else os.path.basename(url.path))
 
         if image_dir:
             self.image_dir = image_dir
@@ -330,37 +338,21 @@ class FileGlanceImageFixture(UploadGlanceImageFixture):
         return os.path.join(self.real_image_dir, self.image_file)
 
     def get_image_data(self):
-        return self.get_image_file(image_file=self.real_image_file)
+        real_image_file = self.real_image_file
+        # if the file exists, then skip the download part
+        if os.path.exists(real_image_file):
+            return self.get_image_from_file(real_image_file)
+        # else, download the image
+        return self.get_image_from_url(real_image_file)
 
-    def get_image_file(self, image_file: str):
-        image_file = self.customize_image_file(base_file=image_file)
+    def get_image_from_file(self, image_file: str):
         image_size = os.path.getsize(image_file)
         LOG.debug('Uploading image %r data from file %r (%d bytes)',
                   self.image_name, image_file, image_size)
-        image_data = _io.open_image_file(
-            filename=image_file, mode='rb',
-            compression_type=self.compression_type)
+        image_data = _io.open_image_file(filename=image_file, mode='rb')
         return image_data, image_size
 
-    def customize_image_file(self, base_file: str) -> str:
-        return base_file
-
-
-class URLGlanceImageFixture(FileGlanceImageFixture):
-
-    image_url: str
-
-    def __init__(self,
-                 image_url: str = None,
-                 **kwargs):
-        super().__init__(**kwargs)
-        if image_url is None:
-            image_url = self.image_url
-        else:
-            self.image_url = image_url
-        tobiko.check_valid_type(image_url, str)
-
-    def get_image_file(self, image_file: str):
+    def get_image_from_url(self, image_file: str):
         # pylint: disable=missing-timeout
         http_request = requests.get(self.image_url, stream=True)
         expected_size = int(http_request.headers.get('content-length', 0))
@@ -394,8 +386,7 @@ class URLGlanceImageFixture(FileGlanceImageFixture):
             self._download_image_file(image_file=image_file,
                                       chunks=chunks,
                                       expected_size=expected_size)
-        return super(URLGlanceImageFixture, self).get_image_file(
-            image_file=image_file)
+        return self.get_image_from_file(image_file=image_file)
 
     def _download_image_file(self, image_file, chunks, expected_size):
         image_dir = os.path.dirname(image_file)
@@ -419,116 +410,11 @@ class URLGlanceImageFixture(FileGlanceImageFixture):
         os.rename(temp_file, image_file)
 
 
-class CustomizedGlanceImageFixture(FileGlanceImageFixture):
-
-    @property
-    def firstboot_commands(self) -> typing.List[str]:
-        return []
-
-    @property
-    def install_packages(self) -> typing.List[str]:
-        return []
-
-    @property
-    def run_commands(self) -> typing.List[str]:
-        return []
-
-    @property
-    def write_files(self) -> typing.Dict[str, str]:
-        return {}
-
-    username: str = ''
-    password: str = ''
-
-    def _get_customized_suffix(self) -> str:
-        return 'customized'
-
-    def customize_image_file(self, base_file: str) -> str:
-
-        def workaround_passt(full_command, exc):
-            which_passt = sh.execute(
-                'which passt', expect_exit_status=None).stdout.rstrip()
-            if which_passt == '':
-                raise exc
-
-            cmd = f'mv {which_passt} {which_passt}.bak'
-            cmd_cleanup = f'mv {which_passt}.bak {which_passt}'
-            tobiko.add_cleanup(sh.execute, cmd_cleanup,
-                               expect_exit_status=None, sudo=True)
-            LOG.exception("Executing virt-customize without passt")
-            sh.execute(cmd, sudo=True)
-            sh.execute(full_command)
-
-        customized_file = f'{base_file}-{self._get_customized_suffix()}'
-        if os.path.isfile(customized_file):
-            if (os.stat(base_file).st_mtime_ns <
-                    os.stat(customized_file).st_mtime_ns):
-                LOG.debug(f"Image file is up to date '{customized_file}'")
-                return customized_file
-            else:
-                LOG.debug(f"Remove obsolete image file '{customized_file}'")
-                os.remove(customized_file)
-        work_file = sh.execute('mktemp').stdout.strip()
-        try:
-            LOG.debug(f"Copy base image file: '{base_file}' to '{work_file}'")
-            sh.put_file(base_file, work_file)
-
-            options = self.get_virt_customize_options()
-            if options:
-                command = sh.shell_command(['LIBGUESTFS_BACKEND=direct',
-                                            'virt-customize',
-                                            '-a',
-                                            work_file])
-                try:
-                    sh.execute(command + options)
-                except sh.ShellCommandFailed as exc:
-                    workaround_passt(command + options, exc)
-
-            sh.get_file(work_file, customized_file)
-            return customized_file
-        finally:
-            sh.execute(['rm', '-f', work_file])
-
-    def get_virt_customize_options(self) -> sh.ShellCommand:
-        options = sh.ShellCommand()
-
-        firstboot_commands = self.firstboot_commands
-        if firstboot_commands:
-            for cmd in firstboot_commands:
-                options += ['--firstboot-command', cmd]
-
-        install_packages = self.install_packages
-        if install_packages:
-            options += ['--install', ','.join(install_packages)]
-
-        username = self.username
-        password = self.password
-        if username and password:
-            options += f'--password "{username}:password:{password}"'
-
-        run_commands = self.run_commands
-        if run_commands:
-            for cmd in run_commands:
-                options += ['--run-command', cmd]
-
-        write_files = self.write_files
-        if write_files:
-            for filename, content in write_files.items():
-                options += ['--write', f'{filename}:{content}']
-
-        return options
-
-
 class InvalidGlanceImageStatus(tobiko.TobikoException):
     message = ("Invalid image {image_name!r} (id {image_id!r}) status: "
                "{actual_status!r} not in {expected_status!r}")
 
 
-class GlanceImageCreationFailed(tobiko.TobikoException):
-    message = ("Failed creating image {image_name!r}: status "
-               "({observed!r}) not in ({expected!r})")
-
-
 CHANGING_STATUS = {GlanceImageStatus.QUEUED,
                    GlanceImageStatus.IMPORTING,
                    GlanceImageStatus.PENDING_DELETE,
